const DateHandler = require('../functions/DateHandler');
const Logger = require('./Logger');

class DataBaseLogger extends Logger {

  constructor() {
    super({
      fileName: 'database'
    });
    this.separator = ''.padEnd(10, '=');
    this.logDateFormat = 'DD/MM/YYYY HH:mm:ss';
    this.template = `${this.separator} [__SYSDATE__]\n__TYPE____ERROR____SQL____PARAMS__\n`;
    this.logFolder = process.env.LOG_FOLDER;
  }

  getErrorMessage(error) {
    if(typeof error === 'string') return error;
    if(error instanceof Error) return `${error.message} ${error.stack}`;
    return `Unexpected Error Structure ${error}`;
  }

  getHandledText({label, value}) {
    if(!value) return '';
    const text = typeof value === 'object' ? JSON.stringify(value) : value;
    return `${label}: ${text}\n`;
  }

  getLogByTemplate({ error, sql, type, params = {} }) {
    let logText = this.template;
    logText = logText.replace('__TYPE__', this.getHandledText({label: 'TYPE', value: type}));
    logText = logText.replace('__ERROR__', this.getHandledText({label: 'ERROR', value: error}));
    logText = logText.replace('__SQL__', this.getHandledText({label: 'SQL', value: sql}));
    logText = logText.replace('__PARAMS__', this.getHandledText({label: 'PARAMS', value: params}));
    logText = logText.replace('__SYSDATE__', DateHandler.toString({ format: this.logDateFormat }));
    return logText;
  }

  format(error, info) {
    const errorStr = this.getErrorMessage(error);
    return this.getLogByTemplate({...info, error: errorStr});
  }

}

module.exports = new DataBaseLogger();