const DB = require('../database/DB');

class UpdaterEventos {
  INSERT = `
    INSERT INTO
    UPDATEREVENTOS (
      CDFILIAL,
      CDEVENTO,
      CDUUID,
      CDPRODUCTID,
      CDPRODFILIID,
      IDSTEVENTO,
      NRVERSAOATUAL,
      NRVERSAOALVO,
      IDCRIARVERSAO,
      DTINCLUSAO,
      DTULTATU
    ) VALUES (
      :P_CDFILIAL,
      :P_CDEVENTO,
      :P_CDUUID,
      :P_CDPRODUCTID,
      :P_CDPRODFILIID,
      :P_IDSTEVENTO,
      :P_NRVERSAOATUAL,
      :P_NRVERSAOALVO,
      :P_IDCRIARVERSAO,
      SYSDATE,
      SYSDATE
    )
  `;

  DELETE_PRODUCT_EVENTS = `
    DELETE FROM
      UPDATEREVENTOS
    WHERE
      CDFILIAL    = :CDFILIAL     AND
      CDPRODUCTID = :CDPRODUCTID
  `;

  UPDATE_EVENT_STATUS = `
    UPDATE UPDATEREVENTOS
    SET
      IDSTEVENTO = :P_IDSTEVENTO,
      DTULTATU   = SYSDATE
    WHERE
      CDEVENTO   = :P_CDEVENTO
  `;

  CANCEL_EVENT = `
    UPDATE
      UPDATEREVENTOS
    SET
      IDSTEVENTO = 'CANCELADO',
      DTULTATU = SYSDATE
    WHERE
      CDFILIAL = :P_CDFILIAL
  `;

  GET_PRODUCT_EVENTS = `
    SELECT
      *
    FROM
      UPDATEREVENTOS
    WHERE
      CDFILIAL = :P_CDFILIAL AND
      IDSTEVENTO = :P_IDSTEVENTO
  `;

  GET_ACTIVE_EVENTS = `
    SELECT
      *
    FROM
      UPDATEREVENTOS
    WHERE
      CDFILIAL = :P_CDFILIAL AND
      IDSTEVENTO NOT IN ('CANCELADO', 'FINALIZADO')
  `;

  GET_EVENTS_IN_PROGRESS = `
    SELECT
      *
    FROM
      UPDATEREVENTOS
    WHERE
      CDFILIAL = :P_CDFILIAL
      AND IDSTEVENTO IN (
        'INSTALACAO_EM_CURSO',
        'REMOCAO_EM_CURSO',
        'REMOCAO_PENDENTE',
        'VERIFICACAO_EM_CURSO',
        'ANALISE_INTEGRIDADE_EM_CURSO',
        'ANALISE_INTEGRIDADE_PENDENTE'
      )
  `;

  GET_PENDING_EVENTS = `
    SELECT
      UE.CDFILIAL,
      UPH.CDOPERADOR,
      UE.CDEVENTO,
      UE.CDPRODUCTID,
      UE.CDPRODFILIID,
      UE.IDSTEVENTO,
      UE.NRVERSAOATUAL,
      UE.NRVERSAOALVO,
      UE.IDCRIARVERSAO,
      UE.DTINCLUSAO,
      UPV.DSDIRETORIO,
      UPV.IDDOWNLOADVER,
      UPC.CONFIG
    FROM
      UPDATEREVENTOS UE
      LEFT JOIN UPDATERPRODVERSAO UPV ON (
        UE.CDPRODUCTID = UPV.UPDATERPRODID AND
        (
          (UE.NRVERSAOALVO IS NOT NULL AND UE.NRVERSAOALVO = UPV.NRVERSAO)
            OR
          (UE.NRVERSAOATUAL IS NOT NULL AND UE.NRVERSAOATUAL = UPV.NRVERSAO)
        )
      )
      LEFT JOIN UPDATERPRODCONFIGFILI UPC ON (
        UE.CDPRODUCTID = UPC.CDPRODUCTID AND
        UE.CDFILIAL = UPC.CDFILIAL AND
        (
          (UE.NRVERSAOALVO IS NOT NULL AND UPC.NRVERSAO = UE.NRVERSAOALVO)
            OR
          (UE.NRVERSAOATUAL IS NOT NULL AND UPC.NRVERSAO = UE.NRVERSAOATUAL)
        )
      )
      LEFT JOIN UPDATERHISTORICO UPH ON (
        UE.CDEVENTO   = UPH.CDEVENTO AND
        UE.IDSTEVENTO = UPH.IDSTEVENTO
      )
    WHERE
      UE.CDFILIAL = :P_CDFILIAL AND
      UE.IDSTEVENTO NOT IN ('CANCELADO', 'FINALIZADO')
  `;

  insert(params, executionParams = {}) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  deleteProductEvents(params, executionParams = {}) {
    return DB.execute(this.DELETE_PRODUCT_EVENTS, params, executionParams);
  }

  updateEventStatus(params, executionParams = {}) {
    return DB.execute(this.UPDATE_EVENT_STATUS, params, executionParams);
  }

  cancelEvent(params, executionParams = {}) {
    return DB.execute(this.CANCEL_EVENT, params, executionParams);
  }

  getProductEvents(params) {
    return DB.fetch(this.GET_PRODUCT_EVENTS, params);
  }


  getActiveEvents(params) {
    return DB.fetch(this.GET_ACTIVE_EVENTS, params);
  }

  getEventsInProgress(params) {
    return DB.fetch(this.GET_EVENTS_IN_PROGRESS, params);
  }

  getPendingEvents(params) {
    return DB.fetch(this.GET_PENDING_EVENTS, params);
  }
}

module.exports = new UpdaterEventos();