const DB = require('../database/DB');

class UpdaterHistoricoCx {
  INSERT = `
    INSERT INTO
    UPDATERHISTORICOCX (
      CDFILIAL,
      CDCAIXA,
      CDHISTORICO,
      CDEVENTO,
      CDOPERADOR,
      CDUUID,
      CDPRODUCTID,
      NMPRODUCT,
      IDSTEVENTO,
      NRVERSAOATUAL,
      NRVERSAOALVO,
      IDCRIARVERSAO,
      DTINCLUSAO,
      DTULTATU
    ) VALUES (
      :P_CDFILIAL,
      :P_CDCAIXA,
      :P_CDHISTORICO,
      :P_CDEVENTO,
      :P_CDOPERADOR,
      :P_CDUUID,
      :P_CDPRODUCTID,
      :P_NMPRODUCT,
      :P_IDSTEVENTO,
      :P_NRVERSAOATUAL,
      :P_NRVERSAOALVO,
      :P_IDCRIARVERSAO,
      SYSDATE,
      SYSDATE
    )
  `;

  DELETE = `
    DELETE FROM
      UPDATERHISTORICOCX
    WHERE
      CDFILIAL    = :CDFILIAL AND
      CDCAIXA     = :CDCAIXA  AND
      CDPRODUCTID = :CDPRODUCTID
  `;

  GET_EVENT_HISTORY = `
    SELECT
      *
    FROM
      UPDATERHISTORICOCX
    WHERE
      CDEVENTO   = :P_CDEVENTO   AND
      IDSTEVENTO = :P_IDSTEVENTO
  `;

  GET_PRODUCT_HISTORY = `
    SELECT
      H.CDFILIAL,
      O.NMOPERADOR,
      F.NMFILIAL,
      H.CDFILIAL
      || ' - '
      || F.NMFILIAL AS FILIAL,
      C.CDCAIXA
      || ' - '
      || C.NMCAIXA AS CAIXA,
      H.CDUUID,
      H.CDPRODUCTID,
      P.NMPRODUCT,
      H.IDSTEVENTO,
      NVL(H.NRVERSAOATUAL, '-') AS NRVERSAOATUAL,
      H.NRVERSAOALVO,
      H.IDCRIARVERSAO,
      TO_CHAR(H.DTULTATU, 'DD/MM/YYYY HH24:MI:SS') AS DTULTATU
    FROM
      UPDATERHISTORICOCX H
      JOIN UPDATERPRODUTOS P ON (
        P.ID = H.CDPRODUCTID
      )
      JOIN FILIAL F ON (
        F.CDFILIAL = H.CDFILIAL
      )
      LEFT JOIN OPERADOR O ON (
        O.CDOPERADOR = H.CDOPERADOR
      )
      INNER JOIN CAIXA C ON (
        C.CDFILIAL = H.CDFILIAL AND
        C.CDCAIXA  = H.CDCAIXA
      )
    WHERE
      H.CDPRODUCTID = :CDPRODUCTID AND
      H.CDFILIAL    = :CDFILIAL    AND
      H.CDCAIXA     = :CDCAIXA
    ORDER BY
      H.DTULTATU DESC
  `;

  insert(params, executionParams) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  delete(params, executionParams) {
    return DB.execute(this.DELETE, params, executionParams);
  }

  getEventHistory(params) {
    return DB.fetch(this.GET_EVENT_HISTORY, params);
  }

  getProductHistory(params) {
    return DB.fetch(this.GET_PRODUCT_HISTORY, params);
  }
}

module.exports = new UpdaterHistoricoCx();