const DB = require("../database/DB");

class UpdaterProdFili {
  INSERT = `
    INSERT INTO UPDATERPRODFILI
      ( UPDATERPRODID,  CDFILIAL, DTINCLUSAO, DTULTATU)
    VALUES
      (:UPDATERPRODID, :CDFILIAL, SYSDATE,    SYSDATE)
  `;

  UPDATE = `
    UPDATE
      UPDATERPRODFILI
    SET
      NRVERSAOATUAL = :NRVERSAO,
      DTULTATU      = SYSDATE
    WHERE
      ID = :ID
  `;

  GET_INSTALLED_PRODUCTS = `
    SELECT
      ID,
      UPDATERPRODID,
      NRVERSAOATUAL
    FROM
      UPDATERPRODFILI
    WHERE
      CDFILIAL = :CDFILIAL
  `;

  GET_INSTALLED_VERSIONS_BY_PRODUCT = `
    SELECT
      ID,
      CDFILIAL,
      NRVERSAOATUAL
    FROM UPDATERPRODFILI
    WHERE
      UPDATERPRODID = :UPDATERPRODID
  `;

  GET_VERSION = `
    SELECT
      ID
    FROM
      UPDATERPRODFILI
    WHERE
      CDFILIAL      = :P_CDFILIAL      AND
      UPDATERPRODID = :P_UPDATERPRODID AND
      NRVERSAOATUAL = :P_NRVERSAOATUAL
  `

  GET_MISSING_REQUIRED_PRODUCTS = `
    SELECT 
      UPF.CDFILIAL,
      UP.ID,
      UP.NMPRODUCT
    FROM 
      UPDATERPRODUTOS UP
      INNER JOIN UPDATERPRODFILI UPF ON ( UP.ID = UPF.UPDATERPRODID )
    WHERE 
      UPF.CDFILIAL = :CDFILIAL 
      AND UPF.UPDATERPRODID IN (SELECT TO_NUMBER(REGEXP_SUBSTR(:REQPRODS, '[^,]+', 1, LEVEL)) FROM DUAL CONNECT BY LEVEL <= LENGTH(:REQPRODS) - LENGTH(REPLACE(:REQPRODS, ',', '')) + 1)
      AND UPF.NRVERSAOATUAL IS NULL
  `;

  insert(params) {
    return DB.execute(this.INSERT, params);
  }

  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }

  getInstalledProducts(params) {
    return DB.fetch(this.GET_INSTALLED_PRODUCTS, params);
  }

  getInstalledVersionsByProduct(params) {
    return DB.fetch(this.GET_INSTALLED_VERSIONS_BY_PRODUCT, params);
  }

  getVersion(params) {
    return DB.fetch(this.GET_VERSION, params);
  }
  
  getMissingRequiredProducts(params) {
    return DB.fetch(this.GET_MISSING_REQUIRED_PRODUCTS, params);
  }
}

module.exports = new UpdaterProdFili();