const DB = require('../database/DB');

class UpdaterProdFiliCx {
  INSERT = `
    INSERT INTO UPDATERPRODFILICX
      ( UPDATERPRODID, CDFILIAL, CDCAIXA )
    VALUES
      ( :UPDATERPRODID, :CDFILIAL, :CDCAIXA )
  `;

  UPDATE = `
    UPDATE
      UPDATERPRODFILICX
    SET
      NRVERSAOATUAL = :P_NRVERSAO,
      DTULTATU      = SYSDATE
    WHERE
      ID = :P_ID
  `;

  GET_VERSION = `
    SELECT
      ID
    FROM
      UPDATERPRODFILICX
    WHERE
      CDFILIAL      = :P_CDFILIAL      AND
      CDCAIXA       = :P_CDCAIXA       AND
      UPDATERPRODID = :P_UPDATERPRODID AND
      NRVERSAOATUAL = :P_NRVERSAOATUAL
  `;

  GET_INSTALLED_PRODUCTS = `
    SELECT
      ID,
      UPDATERPRODID,
      NRVERSAOATUAL
    FROM
      UPDATERPRODFILICX
    WHERE
      CDFILIAL = :CDFILIAL AND
      CDCAIXA = :CDCAIXA
  `;

  GET_MISSING_REQUIRED_PRODUCTS = `
    SELECT 
      UPFCX.CDFILIAL,
      UP.ID,
      UP.NMPRODUCT
    FROM 
      UPDATERPRODUTOS UP
      INNER JOIN UPDATERPRODFILICX UPFCX ON ( UP.ID = UPFCX.UPDATERPRODID )
    WHERE 
      UPFCX.CDFILIAL = :CDFILIAL
      AND UPFCX.CDCAIXA = :CDCAIXA
      AND UPFCX.UPDATERPRODID IN (SELECT TO_NUMBER(REGEXP_SUBSTR(:REQPRODS, '[^,]+', 1, LEVEL)) FROM DUAL CONNECT BY LEVEL <= LENGTH(:REQPRODS) - LENGTH(REPLACE(:REQPRODS, ',', '')) + 1)
      AND UPFCX.NRVERSAOATUAL IS NULL
  `;

  insert(params, executionParams = {}) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }

  getInstalledProducts(params) {
    return DB.fetch(this.GET_INSTALLED_PRODUCTS, params);
  }

  getVersion(params) {
    return DB.fetch(this.GET_VERSION, params);
  }

  getMissingRequiredProducts(params) {
    return DB.fetch(this.GET_MISSING_REQUIRED_PRODUCTS, params);
  }
}

module.exports = new UpdaterProdFiliCx();