const DB = require('../database/DB');

class updaterModel {
  constructor() {

    this.GET_DOWNLOADED_VERSIONS = `
      SELECT
        UPDATERPRODID,
        NRVERSAO
      FROM
        UPDATERPRODVERSAO
      WHERE
        IDDOWNLOADVER = 'S'
    `;

    this.GET_DEFAULT_OPERATOR = `
      SELECT
        CDOPERADOR,
        CDVENDEDOR
      FROM
        PARAVEND
      WHERE
        CDFILIAL = :P_CDFILIAL
    `;

    this.GET_DEFAULT_OPERATOR_FROM_SALESMAN = `
      SELECT
        CDOPERADOR
      FROM
        VENDEDOR
      WHERE
        CDVENDEDOR = :P_CDVENDEDOR
    `;

  this.GET_UPDATE_PRODUCT_LAST_RECORD  = `
    SELECT
      *
    FROM
      UPDATERPRODVERSAO
    WHERE
      UPDATERPRODID = :UPDATERPRODID AND
      DTINCLUSAO IS NOT NULL
      ORDER BY DTINCLUSAO DESC
      FETCH FIRST 1 ROW ONLY
  `;

  this.SET_NEW_VERSION_UPDATERPRODVERSAO = `
    INSERT INTO
      UPDATERPRODVERSAO
        (
          UPDATERPRODID,
          NRVERSAO,
          DSURL,
          DSDIRETORIO,
          IDATIVO,
          IDDOWNLOADVER,
          IDAPROVVER,
          DTINCLUSAO,
          DTULTATU
        )
      VALUES
        (
          :UPDATERPRODID,
          :NRVERSAO,
          :DSURL,
          :DSDIRETORIO,
          :IDATIVO,
          :IDDOWNLOADVER,
          :IDAPROVVER,
           SYSDATE,
           SYSDATE
        )
  `;

  this.UPDATE_PACKAGE_URL = `
    UPDATE
      UPDATERPRODVERSAO
    SET
      DSURL = :DSURL,
      DSDIRETORIO = :DSDIRETORIO,
      DTULTATU = SYSDATE
    WHERE
      UPDATERPRODID = :UPDATERPRODID
      AND NRVERSAO = :NRVERSAO
  `;

  this.GET_PRODUCTS_VERSION = `
    SELECT
      PF.ID AS PRODFILIID,
      TO_CHAR(PF.DTANALISE, 'DD/MM/YYYY HH24:MI') AS DTANALISE,
      PF.STANALISE,
      PF.DSHASHANALISE,
      F.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
      F.CDFILIAL,
      F.NMFILIAL,
      P.ID AS PRODUCTID,
      P.NMPRODUCT,
      CASE
          WHEN PF.NRVERSAOATUAL IS NULL THEN
              'INSTALACAO_PENDENTE'
          ELSE
              'INSTALADO'
      END AS IDSTEVENTO,
      DECODE(PF.NRVERSAOATUAL, NULL, '-', PF.NRVERSAOATUAL) AS NRVERSAOATUAL,
      TO_CHAR(PF.DTULTATU, 'DD/MM/YYYY HH24:MI') AS DTULTATU,
      OP.CDOPERADOR,
      OP.NMOPERADOR,
      UPV.DSDIRETORIO
    FROM
      UPDATERPRODFILI PF
      INNER JOIN UPDATERPRODUTOS  P ON ( PF.UPDATERPRODID = P.ID )
      INNER JOIN FILIAL           F ON ( PF.CDFILIAL = F.CDFILIAL )
      LEFT JOIN UPDATERPRODVERSAO UPV ON (PF.UPDATERPRODID = UPV.UPDATERPRODID AND NVL(PF.NRVERSAOATUAL, '-') = UPV.NRVERSAO)
      LEFT JOIN (
          SELECT
              CDFILIAL,
              CDPRODFILIID,
              CDPRODUCTID,
              MAX(CDEVENTO) AS CDEVENTO
          FROM
              UPDATEREVENTOS
          WHERE
              IDSTEVENTO = 'FINALIZADO'
          GROUP BY
              CDFILIAL,
              CDPRODUCTID,
              CDPRODFILIID
      ) EV
      ON (
              EV.CDPRODUCTID = P.ID
          AND EV.CDPRODFILIID = PF.ID
          AND EV.CDFILIAL = PF.CDFILIAL
      )
      LEFT JOIN UPDATERHISTORICO HIST
          ON  HIST.CDEVENTO = EV.CDEVENTO
          AND HIST.IDSTEVENTO = 'FINALIZADO'
      LEFT JOIN OPERADOR OP
          ON OP.CDOPERADOR = HIST.CDOPERADOR
    WHERE
      PF.CDFILIAL = :P_CDFILIAL
    ORDER BY
      P.NMPRODUCT ASC
  `;

  this.GET_OPERATOR_AND_PRODUCT_NAME = `
    SELECT
      (
        SELECT
          NMOPERADOR
        FROM
          OPERADOR
        WHERE
          CDOPERADOR = :CDOPERADOR
      ) AS NMOPERADOR,
      (
        SELECT
          NMPRODUCT
        FROM
          UPDATERPRODUTOS
        WHERE
          ID = :DEVOPSID
      ) AS NMPRODUCT
    FROM DUAL
  `;

  this.UPDATE_PRODFILI_STATUS = `
    UPDATE
      UPDATERPRODFILI
    SET
      STANALISE = :STATUS,
      DTANALISE = SYSDATE
    WHERE ID = :PRODFILIID
  `;

  this.GET_PRODUCTS_TO_ANALISE_INTEGRITY = `
    SELECT
      UP.ID AS PRODFILIID,
      UP.UPDATERPRODID,
      UP.NRVERSAOATUAL,
      UP.DSHASHANALISE,
      UV.DSDIRETORIO
    FROM
      UPDATERPRODFILI UP
      INNER JOIN UPDATERPRODVERSAO UV
          ON UV.UPDATERPRODID = UP.UPDATERPRODID
          AND UV.NRVERSAO = UP.NRVERSAOATUAL
    WHERE
          UP.CDFILIAL = :CDFILIAL
      AND UP.STANALISE = 'PENDENTE'
  `;

  this.UPDATE_INTEGRITY_HASH = `
    UPDATE
      UPDATERPRODFILI
    SET DSHASHANALISE = :DSHASHANALISE
    WHERE ID = :PRODFILIID
  `;

  this.UPDATE_RESET_INTEGRITY_ANALYZE = `
    UPDATE UPDATERPRODFILI
    SET DSHASHANALISE = NULL,
        DTANALISE = NULL,
        STANALISE = NULL
    WHERE ID = :PRODFILIID
  `;

}

  async getDownloadedVersion() {
    return DB.fetch(this.GET_DOWNLOADED_VERSIONS);
  };

  getCheckIfVersionIsInstalled(params) {
    return DB.fetch(this.ARRIVE_CASHIER_VERSION, params);
  };

  getDefaultOperator(params) {
    return DB.fetch(this.GET_DEFAULT_OPERATOR, params);
  };

  getDefaultOperatorFromSalesman(params) {
    return DB.fetch(this.GET_DEFAULT_OPERATOR_FROM_SALESMAN, params);
  };

  getOperatorAndProductName(params) {
    return DB.fetch(this.GET_OPERATOR_AND_PRODUCT_NAME, params, { maxRows: 1 });
  }

  updatePackageURL(params) {
    return DB.execute(this.UPDATE_PACKAGE_URL, params);
  };

  getUpdateLastRecordProduct(params) {
    return DB.fetch(this.GET_UPDATE_PRODUCT_LAST_RECORD, params);
  }

  setUpdateProdVersion(params) {
    return DB.execute(this.SET_NEW_VERSION_UPDATERPRODVERSAO, params);
  }

  async getProductsVersion(params) {
    return DB.fetch(this.GET_PRODUCTS_VERSION, params);
  }

  async updateProdFiliStatus(params) {
    return DB.execute(this.UPDATE_PRODFILI_STATUS, params);
  }

  async getProductsToIntegrityAnalize(params) {
    return DB.fetch(this.GET_PRODUCTS_TO_ANALISE_INTEGRITY, params);
  }

  async updateIntegrityHash(params) {
    return DB.execute(this.UPDATE_INTEGRITY_HASH, params);
  }

  async updateResetIntegrityAnalyze(params) {
    return DB.execute(this.UPDATE_RESET_INTEGRITY_ANALYZE, params);
  }
}

module.exports = new updaterModel();
