const path = require('path');
const currentPath = path.dirname(process.argv[1]);
const envPath = path.join(currentPath, '.env');

require('dotenv').config({path: envPath});

const app = require('./app/main/app');
const SocketService = require('./app/service/SocketService');
const UpdaterLogger = require('./app/logs/UpdaterLogger');
const DataBaseLogger = require('./app/logs/DataBaseLogger');
const Connection = require('./app/database/oracle/Connection');
const http = require('http').createServer(app);

UpdaterLogger.startCleanupTask();
DataBaseLogger.startCleanupTask();
const port = process.argv[3];

if (port) {
  process.on('warning', e => console.warn(e.stack));
  Connection.connect().finally(() => {
    SocketService.start(http);
    http.listen(port, () => {
      console.log(`Running on ${port}!`);
    });
  });
} else {
  console.log(`Usage: node index.js <DB_NAME> <PORT>

  Arguments:
  DB_NAME   Database Name (or EXTERNAL for local database)
  PORT      Port to up server`);

  process.exit(1);
}